@@ -40,7 +40,10 @@
 #include <sys/types.h>
 #include <sys/time.h>
 #include <time.h>
+
+#ifndef __MINGW__
 #include <pwd.h>
+#endif
 
 /* ---------------------------------------------------------------------- */
 
@@ -234,10 +237,10 @@ html_print_field(FieldType field, int at_end)
 char *
 html_convert_string(char *s)
 {
-  static char html_s[MAX_LINE_LENGTH*2];
-  int p;
+  static char html_s[CL_MAX_LINE_LENGTH*2];
+  int p; /* "pointer" into array */
 
-  if (!s || strlen(s) >(MAX_LINE_LENGTH))
+  if (!s || strlen(s) >(CL_MAX_LINE_LENGTH))
     return NULL;
   
   for (p = 0; *s; s++) {
@@ -277,6 +280,14 @@ html_convert_string(char *s)
 
 /* ---------------------------------------------------------------------- */
 
+/**
+ * Prints a line of text (which will have been previously extracted from a corpus
+ * linked to the present corpus by an a-attribute) within HTML markup.
+ *
+ * @param stream          Destination for the output.
+ * @param attribute_name  The name of the aligned corpus: printed in the leading indicator within the HTML.
+ * @param line            Character data of the line of aligned-corpus data to print. This is treated as opaque.
+ */
 void
 html_print_aligned_line(FILE *stream, 
                         char *attribute_name, 
@@ -285,7 +296,7 @@ html_print_aligned_line(FILE *stream,
   fputc('\n', stream);
 
   if (GlobalPrintOptions.print_tabular) {
-    fprintf(stream, "<TR><TD colspan=4%s><EM><B><EM>--&gt;", 
+    Rprintf("<TR><TD colspan=4%s><EM><B><EM>--&gt;", 
             GlobalPrintOptions.print_wrap   ? "" : " nowrap");
   }
   else 
@@ -293,33 +304,33 @@ html_print_aligned_line(FILE *stream,
 
   html_puts(stream, attribute_name, SUBST_ALL);
   html_puts(stream, ":</EM></B>&nbsp;&nbsp;", 0);
-  html_puts(stream, line, SUBST_ALL);
+  html_puts(stream, line, SUBST_NONE); /* entities have already been escaped */
 
   if (GlobalPrintOptions.print_tabular) 
-    fprintf(stream, "</TR>\n");
+    Rprintf("</TR>\n");
   else
     fputc('\n', stream);
 }
 
 void html_print_context(ContextDescriptor *cd, FILE *stream)
 {
-  char *s;
+  /* char *s; */
 
   fputs("<tr><td nowrap><em>Left display context:</em></td><td nowrap>", stream);
 
   switch(cd->left_type) {
   case CHAR_CONTEXT:
-    fprintf(stream, "%d characters", cd->left_width);
+    Rprintf("%d characters", cd->left_width);
     break;
   case WORD_CONTEXT:
-    fprintf(stream, "%d tokens", cd->left_width);
+    Rprintf("%d tokens", cd->left_width);
     break;
   case STRUC_CONTEXT:
-    fprintf(stream, "%d %s", cd->left_width, 
+    Rprintf("%d %s", cd->left_width, 
             cd->left_structure_name ? cd->left_structure_name : "???");
     break;
   default:
-    s = "error";
+    /* s = "error"; */
     break;
   }
 
@@ -329,17 +340,17 @@ void html_print_context(ContextDescriptor *cd, FILE *stream)
 
   switch(cd->right_type) {
   case CHAR_CONTEXT:
-    fprintf(stream, "%d characters", cd->right_width);
+    Rprintf("%d characters", cd->right_width);
     break;
   case WORD_CONTEXT:
-    fprintf(stream, "%d tokens", cd->right_width);
+    Rprintf("%d tokens", cd->right_width);
     break;
   case STRUC_CONTEXT:
-    fprintf(stream, "%d %s", cd->right_width, 
+    Rprintf("%d %s", cd->right_width, 
             cd->right_structure_name ? cd->right_structure_name : "???");
     break;
   default:
-    s = "error";
+    /* s = "error"; */
     break;
   }
 
@@ -349,7 +360,10 @@ void html_print_context(ContextDescriptor *cd, FILE *stream)
 void html_print_corpus_header(CorpusList *cl, FILE *stream)
 {
   time_t now;
+
+#ifndef __MINGW__
   struct passwd *pwd = NULL;
+#endif
 
   if (GlobalPrintOptions.print_header) {
 
@@ -357,7 +371,7 @@ void html_print_corpus_header(CorpusList *cl, FILE *stream)
     /*   pwd = getpwuid(geteuid()); */
     /* disabled because of incompatibilities between different Linux versions */
     
-    fprintf(stream,
+    Rprintf(
             "<em><b>This concordance was generated by:</b></em><p>\n"
             "<table>\n"
             "<tr><td nowrap><em>User:</em></td><td nowrap>%s (%s)</td></tr>\n"
@@ -366,8 +380,13 @@ void html_print_corpus_header(CorpusList *cl, FILE *stream)
             "<tr><td nowrap> </td><td nowrap>%s</td></tr>\n"
             "<tr><td nowrap><em>Subcorpus:</em></td><td nowrap>%s:%s</td></tr>\n"
             "<tr><td nowrap><em>Number of Matches:</em></td><td nowrap>%d</td></tr>\n",
+#ifndef __MINGW__
             (pwd ? pwd->pw_name : "unknown"),
             (pwd ? pwd->pw_gecos  : "unknown"),
+#else
+            "<unknown>",
+            "<unknown>",
+#endif
             ctime(&now),
             (cl->corpus && cl->corpus->registry_name ? cl->corpus->registry_name : "unknown"),
             (cl->corpus && cl->corpus->name ? cl->corpus->name : "unknown"),
@@ -379,7 +398,7 @@ void html_print_corpus_header(CorpusList *cl, FILE *stream)
     fputs("</table>\n", stream);
     
     if (cl->query_corpus && cl->query_text) {
-      fprintf(stream, "<P><EM>Query text:</EM> <BR>\n<BLOCKQUOTE><CODE>\n%s; %s\n</CODE></BLOCKQUOTE>\n",
+      Rprintf("<P><EM>Query text:</EM> <BR>\n<BLOCKQUOTE><CODE>\n%s; %s\n</CODE></BLOCKQUOTE>\n",
               cl->query_corpus, cl->query_text);
     }
     
@@ -396,12 +415,12 @@ void html_print_output(CorpusList *cl,
 {
   int line, real_line;
   ConcLineField clf[NoField];   /* NoField is largest field code (not used by us) */
-  AttributeList *strucs;
+  /* AttributeList *strucs; */
   PrintDescriptionRecord *pdr;
 
   ParsePrintOptions();
 
-  strucs = cd->printStructureTags;
+  /* strucs = cd->printStructureTags; */
 
   if (GlobalPrintOptions.print_tabular) {
 
@@ -410,7 +429,7 @@ void html_print_output(CorpusList *cl,
     else                
       pdr = &HTMLTabularNowrapPrintDescriptionRecord;
 
-    fprintf(stream, "<HR><TABLE%s>\n",
+    Rprintf("<HR><TABLE%s>\n",
             GlobalPrintOptions.print_border ? " BORDER=1" : "");
   }
   else {
@@ -423,7 +442,7 @@ void html_print_output(CorpusList *cl,
   if ((last >= cl->size) || (last < 0))
     last = cl->size - 1;
 
-  for (line = first; (line <= last) && (!broken_pipe); line++) {
+  for (line = first; (line <= last) && !cl_broken_pipe; line++) {
 
     if (cl->sortidx)
       real_line = cl->sortidx[line];
@@ -515,11 +534,11 @@ html_print_group(Group *group, int expand, FILE *fd)
   last_source_id = -999;
   nr_targets = 0;
 
-  fprintf(fd, "<BODY>\n<TABLE>\n");
+  Rprintf("<BODY>\n<TABLE>\n");
 
-  for (cell = 0; cell < group->nr_cells; cell++) {
+  for (cell = 0; (cell < group->nr_cells) && !cl_broken_pipe; cell++) {
 
-    fprintf(fd, "<TR><TD>");
+    Rprintf("<TR><TD>");
 
     source_id = group->count_cells[cell].s;
     
@@ -529,22 +548,22 @@ html_print_group(Group *group, int expand, FILE *fd)
       nr_targets = 0;
     }
     else {
-      fprintf(fd, "&nbsp;");
+      Rprintf("&nbsp;");
     }
     
     target_id = group->count_cells[cell].t;
     target_s = Group_id2str(group, target_id, 1);
     count     = group->count_cells[cell].freq;
     
-    fprintf(fd, "<TD>");
+    Rprintf("<TD>");
     html_puts(fd, target_s, SUBST_ALL);
 
-    fprintf(fd, "<TD>%d</TR>\n", count);
+    Rprintf("<TD>%d</TR>\n", count);
     
     nr_targets++;
   }
 
-  fprintf(fd, "</TABLE>\n</BODY>\n");
+  Rprintf("</TABLE>\n</BODY>\n");
 }
 
 
